
// ############################################################################

// 'r' steht für 'result'
// 'f' steht für 'function'

function rc0=fc0(w1,w2)
    rc0=1/(l1*l2*(m1+m2*(sin(w1-w2))^2));
endfunction;

function rc1=fc1(w1,w2,p1,p2);
    rc1=p1*p2*sin(w1-w2)*fc0(w1,w2);
endfunction;

function rc2=fc2(w1,w2,p1,p2);
    rc2=(m2*(l2*p1)^2+..
        (m1+m2)*(l1*p2)^2-..
        l1*l2*m2*p1*p2*cos(w1-w2))*sin(2*(w1-w2))*0.5*(fc0(w1, w2))^2;
endfunction;

function rw1dot=fw1dot(w1,w2,p1,p2)
    rw1dot=(1/l1)*fc0(w1, w2)*(l2*p1-l1*p2*cos(w1-w2));
endfunction;

function rw2dot=fw2dot(w1,w2,p1,p2)
    rw2dot=(1/(l2*m2))*fc0(w1, w2)*(l1*(m1+m2)*p2-l2*m2*p1*cos(w1-w2));
endfunction;

function rp1dot=fp1dot(w1,w2,p1,p2)
    rp1dot=(-1)*(m1+m2)*gc*l1*sin(w1)-fc1(w1,w2,p1,p2)+fc2(w1,w2,p1,p2);
endfunction;

function rp2dot=fp2dot(w1,w2,p1,p2)
    rp2dot=(-1)*m2*gc*l2*sin(w2)+fc1(w1,w2,p1,p2)-fc2(w1,w2,p1,p2);
endfunction;

// Funktion für die kinetische Energie
//
function rekin=fekin(w1,w2,p1,p2)
    rekin=0.5*m1*l1^2*fw1dot(w1,w2,p1,p2)^2+..
          0.5*m2*..
          (l1^2*fw1dot(w1,w2,p1,p2)^2+..
           l2^2*fw2dot(w1,w2,p1,p2)^2+..
           2*l1*l2*fw1dot(w1,w2,p1,p2)*fw2dot(w1,w2,p1,p2)*cos(w1-w2));
endfunction;

// Funktion für die potentielle Energie
//
function repot=fepot(w1,w2)
    repot=(-1)*(m1+m2)*gc*l1*cos(w1)-m2*gc*l2*cos(w2)
endfunction

// Funktion für die totale Energie
//
function retot=fetot(w1,w2,p1,p2)
    retot=fekin(w1,w2,p1,p2)+fepot(w1,w2);
endfunction;

function retotdata=fetotdata(phasedata)
    for i=1:size(phasedata,"c")
        w1=phasedata(1,i);
        w2=phasedata(2,i);
        p1=phasedata(3,i);
        p2=phasedata(4,i);
        retotdata(i)=fetot(w1,w2,p1,p2);
    end;
endfunction;

function rposdata=fposdata(phasedata)
    for i=1:size(phasedata,"c")
        w1=phasedata(1,i);
        w2=phasedata(2,i);
        x1=l1*sin(w1);
        y1=(-1)*l1*cos(w1);
        x2=x1+l2*sin(w2);
        y2=y1-l2*cos(w2);
        rposdata(1,i)=x1;
        rposdata(2,i)=y1;
        rposdata(3,i)=x2;
        rposdata(4,i)=y2;
    end;
endfunction;

// #############################################################################

x0=[w1_0;w2_0;w1dot_0;w2dot_0];

// #############################################################################

function xdot=fdp(t, x)
    w1=x(1);
    w2=x(2);
    p1=x(3);
    p2=x(4);
    xdot=fw1dot(w1,w2,p1,p2)*[1;0;0;0]+..
         fw2dot(w1,w2,p1,p2)*[0;1;0;0]+..
         fp1dot(w1,w2,p1,p2)*[0;0;1;0]+..
         fp2dot(w1,w2,p1,p2)*[0;0;0;1];
endfunction;

steps=t_end/steptime;
tdata=linspace(0,t_end,steps);

phasedata=ode(x0,t0,tdata,fdp);

// #############################################################################

funcprot(0);

posdata=fposdata(phasedata);
nbpionts=size(posdata,"c");

xmax=(l1+l2)*1.05;
xmin=(-1)*xmax;
ymax=xmax;
ymin=xmin;

whiteid=xget("white");

my_handle = scf(100001);
clf(my_handle,"reset");

drawlater;
plot2d(0,0,-1,rect=[xmin,ymin,xmax,ymax],frameflag=7);
drawnow();
sleep(3000);

realtimeinit(steptime);
realtime(0);

for i=1:nbpionts

    drawlater;

    if i>1 then

        plot2d(posdata(1,i-1),posdata(2,i-1),-1,rect=[xmin,ymin,xmax,ymax],frameflag=7)
        p = get("hdl");
        p.children.mark_mode = "on";
        p.children.mark_foreground = whiteid;

        plot2d(posdata(3,i-1),posdata(4,i-1),-1,rect=[xmin,ymin,xmax,ymax],frameflag=7)
        p = get("hdl");
        p.children.mark_mode = "on";
        p.children.mark_foreground = whiteid;

        px=[0;posdata(1,i-1)];
        py=[0;posdata(2,i-1)];
        xset("color",whiteid);
        xpoly(px,py);

        px=[posdata(1,i-1);posdata(3,i-1)];
        py=[posdata(2,i-1);posdata(4,i-1)];
        xset("color",whiteid);
        xpoly(px,py);
    end;

    plot2d(posdata(1,i),posdata(2,i),-1,rect=[xmin,ymin,xmax,ymax],frameflag=7);
    plot2d(posdata(3,i),posdata(4,i),-1,rect=[xmin,ymin,xmax,ymax],frameflag=7);

    px=[0;posdata(1,i)];
    py=[0;posdata(2,i)];
    xset("color",5);
    xpoly(px,py);

    px=[posdata(1,i);posdata(3,i)];
    py=[posdata(2,i);posdata(4,i)];
    xset("color",5);
    xpoly(px,py);

    plot2d(0,0,-1,rect=[xmin,ymin,xmax,ymax],frameflag=7);

    realtime(i);

    drawnow();

end;

// #############################################################################



















































